#include <stdio.h>
#include <string.h>
#include "config_stc.h"
#include "OLED128x64.h"
#include "timer_gate.h"

#define R1_VAL          (uint16_t)(500) 	// 555电路 R1 Ω
#define R2_VAL          (uint16_t)(500) 	// 555电路 R2 Ω
#define PRESCALER_DIV   (uint16_t)(12)      // Timer1 预分频系数

volatile uint8_t TestFinished = 0;
volatile uint16_t TM0OvfCnt = 0;
volatile uint16_t TallCnt = 0;

void Global_Init(void);
void Delay500ms(void);
double Calc_Capacitance(double Freq);

void main(void)
{
    volatile uint16_t TonCnt = 0;
    volatile double PWM_Freq = 0.0;
    volatile double PWM_Duty = 0.0;
    volatile double Capac = 0;
    volatile char sbuf[22] = {0};
    
    // 0.全局初始化
    Global_Init();

    // 1.OLED 初始化并清屏
	OLED_config();
	printf_ascii_10x24(40,5,"  ");
	FillAll(0);

    memset(sbuf, sizeof(sbuf), 0);
    sprintf(sbuf, "[v1]PWM Testing...");
    printf_ascii_6x8(0, 0, sbuf);
    
    // 2.测高电平计数TonCnt(查询法)
    Timer1_Gate_Init();
    while (INT1);       // 当P3.3为高电平，等待
    Timer1_Gate_Start();  // 当P3.3为低电平时，启动计数器（遇上升沿会自动开始计数）
    while (!INT1);      // 当P3.3为低电平时，再等待
    while (INT1);       // 当P3.3为高电平时，继续等待
    Timer1_Gate_Stop();   // 当P3.3为低电平时，关闭计数器（波形周期计数结束）

    TonCnt = (uint8_t)(TH1);
    TonCnt = (TonCnt << 8) + (uint8_t)(TL1);

    // 3.测周期TperiodCnt计数值（中断法）
    Timer0_Cnt_Init();
    Timer1_IC_Init();
    Timer1_IC_Start();
    
	while(1)
	{  
        Delay500ms();
        
        if(TestFinished)
        {
            // 4.计算结果
            PWM_Freq = ((double)MAIN_Fosc / 12) / TallCnt;
            PWM_Duty = ((double)100 * TonCnt) / TallCnt;
            Capac = Calc_Capacitance(PWM_Freq);

            // 5.显示结果
            memset(sbuf, sizeof(sbuf), 0);
            sprintf(sbuf, "PWM Freq[Hz]%.2f", PWM_Freq);
            printf_ascii_6x8(0, 2, sbuf);
            
            memset(sbuf, sizeof(sbuf), 0);
            sprintf(sbuf, "PWM Duty[ %%]%0.f", PWM_Duty);
            printf_ascii_6x8(0, 3, sbuf);
            
            memset(sbuf, sizeof(sbuf), 0);
            sprintf(sbuf, "   Capac[uF]%0.2f", Capac);
            printf_ascii_6x8(0, 4, sbuf);

            memset(sbuf, sizeof(sbuf), 0);
            sprintf(sbuf, "Tall[%u]Ton[%u]", TallCnt, TonCnt);
            printf_ascii_6x8(0, 5, sbuf);
            
            TestFinished = 0;
        }
	}
}

double Calc_Capacitance(double Freq)
{
	double Capacitance = 1000000.0 * (1.44 / (Freq * (R1_VAL + 2 * R2_VAL)));	// uF as Default
    return Capacitance;
}  

void Global_Init(void)
{
    P_SW2 |= 0x80;  // 使能访问XFR，没有冲突不用关闭
    
    P0M0 = 0x00;
    P0M1 = 0x00;
    P1M0 = 0x00;
    P1M1 = 0x00;
    P2M0 = 0x00;
    P2M1 = 0x00;
    P3M0 = 0x00;
    P3M1 = 0x00;
    P4M0 = 0x00;
    P4M1 = 0x00;
    P5M0 = 0x00;
    P5M1 = 0x00;
}

void Delay500ms(void)	//@12.000MHz
{
	unsigned char data i, j, k;

	_nop_();
	i = 31;
	j = 113;
	k = 29;
	do
	{
		do
		{
			while (--k);
		} while (--j);
	} while (--i);
}

void TM0_Routine() interrupt 1
{
    ++TM0OvfCnt;
}

void TM1_Routine() interrupt 3
{ 
    static uint8_t FirstIn = 1;
    
    if(FirstIn) // T1 Cnt Ovf
    {
        Timer0_Cnt_Start();
        FirstIn = 0;
    }
    else
    {
        Timer0_Cnt_Stop();
        Timer1_IC_Stop();
        
        TallCnt = TH0;
        TallCnt = (TallCnt << 8) + TL0;
        TallCnt += (65536 * TM0OvfCnt);
        
        TestFinished = 1;
    }
}

